__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------


import StringIO
from rdkit.Chem.Draw import SimilarityMaps
from chembl_beaker.beaker.utils.io import _parseSMILESData, _parseMolData

try:
    import matplotlib
    matplotlib.use('Agg')
    from matplotlib import pyplot
except:
    matplotlib=None

# ----------------------------------------------------------------------------------------------------------------------


def _similarityMap(ms, width=500, height=500, radius=2, fingerprint='morgan', format='png'):
    if matplotlib is None:
        raise ValueError('matplotlib not useable')

    fn = None
    if fingerprint == 'morgan':
        fn = lambda x, i: SimilarityMaps.GetMorganFingerprint(x, i, radius=radius)
    elif fingerprint == 'tt':
        fn = SimilarityMaps.GetAPFingerprint
    elif fingerprint == 'ap':
        fn = SimilarityMaps.GetTTFingerprint

    fig, maxv = SimilarityMaps.GetSimilarityMapForFingerprint(ms[0], ms[1], fn, size=(width, height))
    sio = StringIO.StringIO()
    pyplot.savefig(sio, format=format, bbox_inches='tight', dpi=100)

    return sio.getvalue()

# ----------------------------------------------------------------------------------------------------------------------


def _smiles2SimilarityMap(data, width=500, height=500, radius=2, fingerprint='morgan', computeCoords=False,
                          delimiter=' ', smilesColumn=0, nameColumn=1, titleLine=True, sanitize=True, format='png'):
    return _similarityMap(_parseSMILESData(data, computeCoords=computeCoords, delimiter=delimiter,
        smilesColumn=smilesColumn, nameColumn=nameColumn, titleLine=titleLine, sanitize=sanitize),
        width=width, height=height, radius=radius, fingerprint=fingerprint, format=format)


# ----------------------------------------------------------------------------------------------------------------------

def _sdf2SimilarityMap(data, width=500, height=500, radius=2, fingerprint='morgan', sanitize=True, removeHs=True,
                       strictParsing=True, format='png'):
    return _similarityMap(_parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing),
        width=width, height=height, radius=radius, fingerprint=fingerprint, format=format)

# ----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from bottle import request
import base64
import os
from chembl_beaker.beaker import app, config
from chembl_beaker.beaker.core_apps.osra.impl import _image2ctab, _image2smiles

#-----------------------------------------------------------------------------------------------------------------------

def image2ctabView(img):
    known_location = '/usr/local/bin/osra'
    if not os.path.exists(known_location):
        known_location = '/usr/bin/osra'
    return _image2ctab(img, config.get('osra_binaries_location', known_location))

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/image2ctab/<image>', method=['OPTIONS', 'GET'], name="image2ctab")
def image2ctab(image):
    """
Uses OSRA to convert image to CTAB. Image should be urlsafe_base64 encoded data of 300 DPI png graphic.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}image2ctab/$(cat mol.jpg | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}image2ctab/$(cat mol.png | base64 -w 0 | tr "+/" "-_")
    """
    if image.startswith('data:'):
        img = base64.urlsafe_b64decode(image[image.find(',')+1:])
    else:
        img = base64.urlsafe_b64decode(image)
    return image2ctabView(img)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/image2ctab', method=['OPTIONS', 'POST'], name="image2ctab")
def image2ctab():
    """
Uses OSRA to convert image to CTAB. Image should be 300 DPI png graphic.
cURL examples:

    curl -X POST --data-binary @mol.jpg ${BEAKER_ROOT_URL}image2ctab
    curl -X POST -F "file=@mol.jpg" ${BEAKER_ROOT_URL}image2ctab
    curl -X POST --data-binary @mol.png ${BEAKER_ROOT_URL}image2ctab
    curl -X POST -F "file=@mol.png" ${BEAKER_ROOT_URL}image2ctab
    """
    img = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    if img.startswith('data:'):
        img = base64.b64decode(img[img.find(',')+1:])
    return image2ctabView(img)

#-----------------------------------------------------------------------------------------------------------------------

def image2smilesView(img):
    known_location = '/usr/local/bin/osra'
    if not os.path.exists(known_location):
        known_location = '/usr/bin/osra'
    return _image2smiles(img, config.get('osra_binaries_location', known_location))


#-----------------------------------------------------------------------------------------------------------------------

@app.route('/image2smiles/<image>', method=['OPTIONS', 'GET'], name="image2smiles")
def image2smiles(image):
    """
Uses OSRA to convert image to CTAB. Image should be urlsafe_base64 encoded data of 300 DPI png graphic.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}image2smiles/$(cat mol.jpg | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}image2smiles/$(cat mol.png | base64 -w 0 | tr "+/" "-_")
    """
    if image.startswith('data:'):
        img = base64.urlsafe_b64decode(image[image.find(',')+1:])
    else:
        img = base64.urlsafe_b64decode(image)
    return image2smilesView(img)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/image2smiles', method=['OPTIONS', 'POST'], name="image2smiles")
def image2smiles():
    """
Uses OSRA to convert image to CTAB. Image should be 300 DPI png graphic.
cURL examples:

    curl -X POST --data-binary @mol.jpg ${BEAKER_ROOT_URL}image2smiles
    curl -X POST -F "file=@mol.jpg" ${BEAKER_ROOT_URL}image2smiles
    curl -X POST --data-binary @mol.png ${BEAKER_ROOT_URL}image2smiles
    curl -X POST -F "file=@mol.png" ${BEAKER_ROOT_URL}image2smiles
    """

    img = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    if img.startswith('data:'):
        img = base64.b64decode(img[img.find(',')+1:])
    return image2smilesView(img)

#-----------------------------------------------------------------------------------------------------------------------


__author__ = 'mnowotka'

#-----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker import app
from bottle import request, response
from chembl_beaker.beaker.utils.io import _parseFlag
from chembl_beaker.beaker.core_apps.standarisation.impl import _break_bonds, _neutralise, _rules, _unsalt, _standardise
import base64

#-----------------------------------------------------------------------------------------------------------------------

def breakBondsView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    return _break_bonds(data, **kwargs)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/breakBonds/<ctab>', method=['OPTIONS', 'GET'], name="breakBonds")
def breakBonds(ctab):
    """
Break covalent bonds between oxygen or nitrogen atoms and Group I and II metal atoms.
Examples and documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/01\_break\_bonds.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/01_break_bonds.html)
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}breakBonds/$(cat breakBonds.mol | base64 -w 0 | tr "+/" "-_")

    """

    data = base64.urlsafe_b64decode(ctab)
    return breakBondsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/breakBonds', method=['OPTIONS', 'POST'], name="breakBonds")
def breakBonds():
    """
Break covalent bonds between oxygen or nitrogen atoms and Group I and II metal atoms.
Examples and documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/01\_break\_bonds.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/01_break_bonds.html)
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @breakBonds.mol ${BEAKER_ROOT_URL}breakBonds
    curl -X POST -F "file=@breakBonds.mol" ${BEAKER_ROOT_URL}breakBonds
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return breakBondsView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def neutraliseView(data, params):

    kwargs = dict()
    kwargs['balance'] = int(params.get('balance', False))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))

    return _neutralise(data, **kwargs)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/neutralise/<ctab>', method=['OPTIONS', 'GET'], name="neutralise")
def neutralise(ctab):
    """
Neutralise charges.
Examples and documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/02\_neutralise.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/02_neutralise.html)
CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}neutralise/$(cat neutralise.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return neutraliseView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/neutralise', method=['OPTIONS', 'POST'], name="neutralise")
def neutralise():
    """
Neutralise charges.
Examples and documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/02\_neutralise.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/02_neutralise.html)
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @neutralise.mol ${BEAKER_ROOT_URL}neutralise
    curl -X POST -F "file=@neutralise.mol" ${BEAKER_ROOT_URL}neutralise
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return neutraliseView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def unsaltView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    return _unsalt(data, **kwargs)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/unsalt/<ctab>', method=['OPTIONS', 'GET'], name="unsalt")
def unsalt(ctab):
    """
Remove counterions and solvate components.
Examples and documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/04\_unsalt.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/04_unsalt.html)
CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}unsalt/$(cat unsalt.mol | base64 -w 0 | tr "+/" "-_")

    """

    data = base64.urlsafe_b64decode(ctab)
    return unsaltView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/unsalt', method=['OPTIONS', 'POST'], name="unsalt")
def unsalt():
    """
Remove salt/solvates.
Examples and documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/04\_unsalt.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/04_unsalt.html)
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @unsalt.mol ${BEAKER_ROOT_URL}unsalt
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}unsalt
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return unsaltView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def rulesView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    return _rules(data, **kwargs)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/rules/<ctab>', method=['OPTIONS', 'GET'], name="rules")
def rules(ctab):
    """
Apply structure-normalisation transformations.
List of rules and further documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/03\_rules.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/03_rules.html)
CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}rules/$(cat rules.mol | base64 -w 0 | tr "+/" "-_")

    """

    data = base64.urlsafe_b64decode(ctab)
    return rulesView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/rules', method=['OPTIONS', 'POST'], name="rules")
def rules():
    """
Apply structure-normalisation transformations.
List of rules and further documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/03\_rules.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/03_rules.html)
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @rules.mol ${BEAKER_ROOT_URL}rules
    curl -X POST -F "file=@rules.mol" ${BEAKER_ROOT_URL}rules
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return rulesView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

def standardiseView(data, params):

    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    return _standardise(data, **kwargs)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/standardise/<ctab>', method=['OPTIONS', 'GET'], name="standardise")
def standardise(ctab):
    """
Get standardised parent.
Examples and documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/05\_standardise.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/05_standardise.html)
CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}standardise/$(cat standardise.mol | base64 -w 0 | tr "+/" "-_")

    """

    data = base64.urlsafe_b64decode(ctab)
    return standardiseView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------

@app.route('/standardise', method=['OPTIONS', 'POST'], name="standardise")
def standardise():
    """
Get standardised parent.
Examples and documentation: [https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/05\_standardise.html](https://wwwdev.ebi.ac.uk/chembl/extra/francis/standardiser/05_standardise.html)
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @standardise.mol ${BEAKER_ROOT_URL}standardise
    curl -X POST -F "file=@standardise.mol" ${BEAKER_ROOT_URL}standardise
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return standardiseView(data, request.params)

#-----------------------------------------------------------------------------------------------------------------------


__author__ = 'mnowotka'

from rdkit import Chem
from standardiser.break_bonds import run as break_bonds
from standardiser.neutralise import run as neutralise
from standardiser.rules import run as rules
from standardiser.unsalt import is_nonorganic, is_salt
from standardiser.standardise import run as standardise
from chembl_beaker.beaker.utils.functional import _apply
from chembl_beaker.beaker.utils.io import _parseMolData, _getSDFString

#-----------------------------------------------------------------------------------------------------------------------

def unsalt(mol):
    non_salt_frags = []
    for n, frag in enumerate(Chem.GetMolFrags(mol, asMols=True), 1):
        if is_nonorganic(frag) or is_salt(frag):
            continue
        non_salt_frags.append(frag)

    if len(non_salt_frags) != 1:
        return []

    return non_salt_frags[0]

#-----------------------------------------------------------------------------------------------------------------------

def _break_bonds(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    res = _apply(mols, break_bonds)
    return _getSDFString(res)

#-----------------------------------------------------------------------------------------------------------------------

def _neutralise(data, balance, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    res = _apply(mols, neutralise, balance)
    return _getSDFString(res)

#-----------------------------------------------------------------------------------------------------------------------

def _rules(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    res = _apply(mols, rules)
    return _getSDFString(res)

#-----------------------------------------------------------------------------------------------------------------------

def _unsalt(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    res = _apply(mols, unsalt)
    return _getSDFString(res)

#-----------------------------------------------------------------------------------------------------------------------

def _standardise(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    res = _apply(mols, standardise)
    return _getSDFString(res)

#-----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker import app
from chembl_beaker.beaker.core_apps.mcs.impl import _mcs
from chembl_beaker.beaker.utils.io import _parseFlag
from bottle import request
import base64

# ----------------------------------------------------------------------------------------------------------------------


def mcsView(data, params):

    kwargs = dict()
    kwargs['asSmiles'] = _parseFlag(params.get('asSmiles', '0'))
    kwargs['atomCompare'] = params.get('atomCompare', 'elements')
    kwargs['bondCompare'] = params.get('bondCompare', 'bondtypes')
    kwargs['ringMatchesRingOnly'] = _parseFlag(params.get('ringMatchesRingOnly', False))
    kwargs['completeRingsOnly'] = _parseFlag(params.get('completeRingsOnly', False))
    kwargs['threshold'] = params.get('threshold', None)
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['isomericSmiles'] = _parseFlag(params.get('isomericSmiles', False))
    kwargs['canonical'] = _parseFlag(params.get('canonical', True))
    kwargs['kekuleSmiles'] = _parseFlag(params.get('kekuleSmiles', False))

    return _mcs(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/mcs/<ctab>', method=['OPTIONS', 'GET'], name="mcs")
def mcs(ctab):
    """
Returns Maximum Common Substructure of a set of compounds. CTAB is urlsafe_base64 encoded string containing molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}mcs/$(cat mcs.sdf | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return mcsView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/mcs', method=['OPTIONS', 'POST'], name="mcs")
def mcs():
    """
Returns Maximum Common Substructure of a set of compounds. This method accepts compounds in SDF format.
cURL examples:

    curl -X POST --data-binary @mcs.sdf ${BEAKER_ROOT_URL}mcs
    curl -X POST -F "file=@mcs.sdf" ${BEAKER_ROOT_URL}mcs
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return mcsView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from rdkit.Chem import Descriptors
try:
    from rdkit.Chem.rdMolDescriptors import CalcMolFormula, CalcNumRotatableBonds
except:
    pass
from chembl_beaker.beaker.utils.functional import _call, _apply
from chembl_beaker.beaker.utils.io import _parseMolData

# ----------------------------------------------------------------------------------------------------------------------


def _desc(mol, name):
    if name and isinstance(name, basestring) and hasattr(Descriptors, name):
        return getattr(Descriptors, name)(mol)

# ----------------------------------------------------------------------------------------------------------------------


def _desc_list(mol, names):
    descriptors = dict()
    for name, fn in Descriptors.descList:
        if not names or name in names:
            descriptors[name] = fn(mol)
    if 'MolecularFormula' not in descriptors:
        descriptors['MolecularFormula'] = CalcMolFormula(mol)
        descriptors['NumRotatableBonds'] = CalcNumRotatableBonds(mol)
    return descriptors

# ----------------------------------------------------------------------------------------------------------------------


def _getNumAtoms(data, sanitize=True, removeHs=True, strictParsing=True):
    return _call(_parseMolData(data, sanitize=sanitize, removeHs=removeHs,
                               strictParsing=strictParsing), "GetNumAtoms")

# ----------------------------------------------------------------------------------------------------------------------


def _getNumBonds(data, sanitize=True, removeHs=True, strictParsing=True):
    return _call(_parseMolData(data, sanitize=sanitize, removeHs=removeHs,
                               strictParsing=strictParsing), "GetNumBonds")

# ----------------------------------------------------------------------------------------------------------------------


def _getLogP(data, sanitize=True, removeHs=True, strictParsing=True):
    return _apply(_parseMolData(data, sanitize=sanitize, removeHs=removeHs,
                                strictParsing=strictParsing), _desc, 'MolLogP')

# -----------------------------------------------------------------------------------------------------------------------


def _getTPSA(data, sanitize=True, removeHs=True, strictParsing=True):
    return _apply(_parseMolData(data, sanitize=sanitize, removeHs=removeHs,
                                strictParsing=strictParsing), _desc, 'TPSA')

# ----------------------------------------------------------------------------------------------------------------------


def _getMolWt(data, sanitize=True, removeHs=True, strictParsing=True):
    return _apply(_parseMolData(data, sanitize=sanitize, removeHs=removeHs,
                                strictParsing=strictParsing), _desc, 'MolWt')

# ----------------------------------------------------------------------------------------------------------------------


def _getDescriptors(data, ds, sanitize=True, removeHs=True, strictParsing=True):
    if ds:
        ds = ds.split(',')
    return _apply(_parseMolData(data, sanitize=sanitize, removeHs=removeHs,
                                strictParsing=strictParsing), _desc_list, ds)

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from rdkit import Chem
from chembl_beaker.beaker.utils.functional import _apply
from chembl_beaker.beaker.utils.io import _parseMolData, _parseSMILESData, _getSMILESString, _getSDFString
from chembl_beaker.beaker.utils.io import _getSMARTSString, _getXYZ
from chembl_beaker.beaker.utils.chemical_transformation import _computeCoords
from chembl_beaker.beaker.utils.chemical_transformation import _sanitize
from chembl_beaker.beaker.utils.io import _molFromSmarts

# ----------------------------------------------------------------------------------------------------------------------


def _canonicalize_smiles(data, computeCoords=False, in_delimiter=' ', smilesColumn=0, nameColumn=1, titleLine=True,
                         sanitize=True, out_delimiter=' ', nameHeader='Name', includeHeader=True, isomericSmiles=False,
                         kekuleSmiles=False):
    return _getSMILESString(_parseSMILESData(data, computeCoords=computeCoords, delimiter=in_delimiter,
                                             smilesColumn=smilesColumn, nameColumn=nameColumn, titleLine=titleLine,
                                             sanitize=sanitize),
                            delimiter=out_delimiter, nameHeader=nameHeader, includeHeader=includeHeader,
                            isomericSmiles=isomericSmiles, kekuleSmiles=kekuleSmiles)

# ----------------------------------------------------------------------------------------------------------------------


def _ctab2smarts(data, sanitize=True, removeHs=True, strictParsing=True, isomericSmiles=False):
    return _getSMARTSString(_parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing),
                            isomericSmiles=isomericSmiles)

# ----------------------------------------------------------------------------------------------------------------------


def _ctab2smiles(data, sanitize=True, removeHs=True, strictParsing=True, delimiter=' ', nameHeader='Name',
                 includeHeader=True, isomericSmiles=False, kekuleSmiles=False):
    return _getSMILESString(_parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing),
                            delimiter=delimiter, nameHeader=nameHeader, includeHeader=includeHeader,
                            isomericSmiles=isomericSmiles, kekuleSmiles=kekuleSmiles)

# ----------------------------------------------------------------------------------------------------------------------


def _smiles2ctab(data, computeCoords=True, delimiter=' ', smilesColumn=0, nameColumn=1, titleLine=True, sanitize=True):
    return _getSDFString(_parseSMILESData(data, computeCoords=computeCoords, delimiter=delimiter,
                                          smilesColumn=smilesColumn, nameColumn=nameColumn, titleLine=titleLine,
                                          sanitize=sanitize))

# ----------------------------------------------------------------------------------------------------------------------


def _smarts2ctab(data, computeCoords=True, delimiter=' ', sanitize=True):
    mols = []
    for line in data.splitlines():
        if not line:
            continue
        for chunk in line.strip().split(delimiter):
            if not chunk:
                continue
            mols.append(_molFromSmarts(chunk))
    if computeCoords:
        _apply(mols, _computeCoords, True)
    if sanitize:
        _apply(mols, _sanitize)
    return _getSDFString(mols)

# ----------------------------------------------------------------------------------------------------------------------


def _ctab2xyz(data, computeCoords=True):
    return _getXYZ(_parseMolData(data), computeCoords)

# ----------------------------------------------------------------------------------------------------------------------


def _inchi2ctab(inchis):
    mols = _apply(inchis.split(),Chem.MolFromInchi)
    _apply(mols, _computeCoords)
    return _getSDFString(mols)

# ----------------------------------------------------------------------------------------------------------------------


def _ctab2inchi(data, sanitize=True, removeHs=True, strictParsing=True):
    return '\n'.join(_apply(_parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing),
                            Chem.MolToInchi))

# ----------------------------------------------------------------------------------------------------------------------


def _ctab2inchiKey(data, sanitize=True, removeHs=True, strictParsing=True):
    inchis = _ctab2inchi(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    return _inchi2inchiKey(inchis)

# ----------------------------------------------------------------------------------------------------------------------


def _smiles2inchi(data, computeCoords=False, delimiter=' ', smilesColumn=0, nameColumn=1, titleLine=True,
                  sanitize=True):
    return '\n'.join(_apply(_parseSMILESData(data, computeCoords=computeCoords, delimiter=delimiter,
                                             smilesColumn=smilesColumn, nameColumn=nameColumn, titleLine=titleLine,
                                             sanitize=sanitize),
                            Chem.MolToInchi))

# ----------------------------------------------------------------------------------------------------------------------


def _smiles2inchiKey(data, computeCoords=False, delimiter=' ', smilesColumn=0, nameColumn=1, titleLine=True,
                     sanitize=True):
    inchis = _smiles2inchi(data, computeCoords=computeCoords, delimiter=delimiter, smilesColumn=smilesColumn,
                           nameColumn=nameColumn, titleLine=titleLine, sanitize=sanitize)
    return _inchi2inchiKey(inchis)

# ----------------------------------------------------------------------------------------------------------------------


def _inchi2inchiKey(inchis):
    return '\n'.join([Chem.InchiToInchiKey(inch) for inch in inchis.split()])

# ----------------------------------------------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
# Author Karol Sikora <karol.sikora@laboratorium.ee>, (c) 2012
# Author Michal Nowotka <mmmnow@gmail.com>, (c) 2013-2014

try:
    import cPickle as pickle
except ImportError:
    import pickle
import base64
import pymongo
from datetime import datetime, timedelta
from chembl_beaker.beaker.cache.backends.base import BaseCache
import zlib
import logging
import json
from chembl_beaker.beaker import config

# ----------------------------------------------------------------------------------------------------------------------

MAX_SIZE = 16000000

# ----------------------------------------------------------------------------------------------------------------------


class MongoDBCache(BaseCache):

    def __init__(self):
        super(MongoDBCache, self).__init__()
        self._host = config.get('mongo_host', 'localhost')
        self._port = int(config.get('mongo_port', 27017))
        self._database = config.get('mongo_db', 'beaker_cache')
        self._rshosts = config.get('mongo_rshosts')
        self._rsname = config.get('mongo_rsname')
        self._user = config.get('mongo_user', None)
        self._compression = config.get('mongo_compression', False)
        self._server_selection_timeout_ms = config.get('mongo_server_selection_timeout_ms', 30000)
        self._password = config.get('mongo_pass', None)
        self._socket_timeout_ms = config.get('mongo_socket_timeout_ms', None)
        self._max_time_ms = config.get('mongo_max_time_ms', 2000)
        self._connect_timeout_ms = config.get('mongo_connect_timeout_ms', 20000)
        self.compression_level = config.get('mongo_compression_level', 6)
        self._tag_sets = json.loads(config.get('mongo_tag_sets', '[]'))
        self._read_preference = config.get("mongo_read_preference")
        self._collection = config.get('mongo_collection', 'cache')
        self.log = logging.getLogger(__name__)

# ----------------------------------------------------------------------------------------------------------------------

    def add(self, key, value):
        key = self.make_key(key)
        self._base_set('add', key, value)

# ----------------------------------------------------------------------------------------------------------------------

    def set(self, key, value):
        key = self.make_key(key)
        self._base_set('set', key, value)

# ----------------------------------------------------------------------------------------------------------------------

    def _base_set(self, mode, key, value, **_):
        coll = self._get_collection()
        encoded = self._encode(value)
        document_size = len(encoded)
        data = coll.find_one({'_id': key})
        if data and (mode == 'set' or mode == 'add'):
            pass
        if document_size <= MAX_SIZE:
            coll.insert_one({'_id': key, 'data': encoded})
        else:
            chunks = []
            for i in xrange(0, document_size, MAX_SIZE):
                chunk = encoded[i:i+MAX_SIZE]
                aux_key = self.make_key(chunk)
                coll.insert_one({'_id': aux_key, 'data': chunk})
                chunks.append(aux_key)
            coll.insert_one({'_id': key, 'chunks': chunks})

# ----------------------------------------------------------------------------------------------------------------------

    def _decode(self, data):
        return pickle.loads(zlib.decompress(base64.decodestring(data)))

# ----------------------------------------------------------------------------------------------------------------------

    def _encode(self, data):
        return base64.encodestring(zlib.compress(pickle.dumps(data, pickle.HIGHEST_PROTOCOL), self.compression_level))

# ----------------------------------------------------------------------------------------------------------------------

    def get(self, key, default=None):
        coll = self._get_collection()
        key = self.make_key(key)
        data = coll.find_one({'_id': key}, max_time_ms=self._max_time_ms)
        if not data:
            return default
        raw = data.get('data')
        if not raw:
            chunks = data.get('chunks')
            if chunks:
                raw = ''
                for chunk in chunks:
                    raw += coll.find_one({'_id': chunk}, max_time_ms=self._max_time_ms)['data']
            else:
                return default
        return self._decode(raw)

# ----------------------------------------------------------------------------------------------------------------------

    def delete(self, key):
        key = self.make_key(key)
        coll = self._get_collection()
        data = coll.find_one({'_id': key})
        if data:
            self._delete([key] + data.get('chunks', []))

# ----------------------------------------------------------------------------------------------------------------------

    def _delete(self, ids_to_remove):
        coll = self._get_collection()
        coll.remove({'_id': {'$in': ids_to_remove}})

# ----------------------------------------------------------------------------------------------------------------------

    def clear(self):
        pass

# ----------------------------------------------------------------------------------------------------------------------

    def _cull(self):
        pass

# ----------------------------------------------------------------------------------------------------------------------

    def _get_collection(self):
        if not getattr(self, '_coll', None):
            self._initialize_collection()
        return self._coll

# ----------------------------------------------------------------------------------------------------------------------

    def _initialize_collection(self):
        try:
            from gevent import monkey
            monkey.patch_socket()
        except ImportError:
            pass


        self.connection = pymongo.MongoClient(connect=False,
                                              host=self._host,
                                              port=self._port,
                                              replicaset=self._rsname,
                                              sockettimeoutms=self._socket_timeout_ms,
                                              connecttimeoutms=self._connect_timeout_ms,
                                              serverselectiontimeoutms=self._server_selection_timeout_ms,
                                              read_preference=getattr(pymongo.ReadPreference,
                                                                      self._read_preference,
                                                                      'PRIMARY'))

        self._db = self.connection[self._database]
        if self._user and self._password:
            self._db.authenticate(self._user, self._password)
        if pymongo.version_tuple[0] < 3:
            self._coll = self._db[self._collection]
        else:
            self._coll = self._db.get_collection(self._collection)
            if not self._coll:
                if self._compression:
                    self._coll = self._db.create_collection(self._collection,
                                                            storageEngine={
                                                                'wiredTiger': {'configString': 'block_compressor=none'}
                                                            })
                else:
                    self._coll = self._db.create_collection(self._collection)

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

import chembl_beaker.beaker.utils.chemical_transformation as ct
from chembl_beaker.beaker.utils.functional import _apply
from chembl_beaker.beaker.utils.io import _parseMolData, _getSDFString

# ----------------------------------------------------------------------------------------------------------------------


def _kekulize(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    _apply(mols, ct._kekulize)
    return _getSDFString(mols)

# ----------------------------------------------------------------------------------------------------------------------


def _sanitize(data, sanitizeOps=ct.SANITIZE_ALL):
    mols = _parseMolData(data, sanitize=False, removeHs=False, strictParsing=False)
    try:
        _apply(mols, ct._sanitize, sanitizeOps=sanitizeOps)
    except:
        pass
    return _getSDFString(mols)

# ----------------------------------------------------------------------------------------------------------------------


def _addHs(data, explicitOnly=False, addCoords=False):
    mols = _parseMolData(data, sanitize=True, removeHs=False, strictParsing=True)
    mols = _apply(mols, ct._addHs, explicitOnly=explicitOnly, addCoords=addCoords)
    return _getSDFString(mols)

# ----------------------------------------------------------------------------------------------------------------------


def _removeHs(data, implicitOnly=False):
    mols = _parseMolData(data, sanitize=False, removeHs=False, strictParsing=True)
    mols = _apply(mols, ct._removeHs, implicitOnly=implicitOnly)
    return _getSDFString(mols)

# ----------------------------------------------------------------------------------------------------------------------


def _sssr(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    return _apply(mols, ct._sssr)

# ----------------------------------------------------------------------------------------------------------------------


def _symmsssr(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    return [[list(xx) for xx in x] for x in _apply(mols, ct._symmsssr)]

# ----------------------------------------------------------------------------------------------------------------------


def _align(data, template, force=False):
    mols = _parseMolData(data)
    if template:
        pattern = _parseMolData(template)
    else:
        pattern = [mols[0]]
        mols = mols[1:]
    if not pattern or not mols or len(pattern) != 1:
        return 'Wrong arguments'
    aligned_mols = ct._align(mols, pattern[0], force)
    if aligned_mols:
        return _getSDFString(aligned_mols)

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from itertools import cycle, islice
from chembl_beaker.beaker.utils.functional import _apply
from chembl_beaker.beaker.utils.chemical_transformation import _computeCoords, _atomMapNumber, _kekulize
from chembl_beaker.beaker.utils.io import _parseMolData, _parseSMILESData
from chembl_beaker.beaker.utils.io import _getMatches

NEW_RENDER_ENGINE = False

try:
    from rdkit.Chem.Draw import rdMolDraw2D
    NEW_RENDER_ENGINE = True
except:
    pass
import StringIO
from chembl_beaker.beaker import draw

# ----------------------------------------------------------------------------------------------------------------------


def _mols2imageStream(mols, f, format, size, legend, highlightAtomLists=None, kekulize=True):
    labels = [x for x in islice(cycle(legend), len(mols))] if isinstance(legend, (list, tuple)) else \
             [x for x in islice(cycle([legend]), len(mols))]
    legends = [x.GetProp("_Name") if (x.HasProp("_Name") and x.GetProp("_Name")) else labels[idx]
               for idx, x in enumerate(mols)]

    molsPerRow = min(len(mols), 4)
    nRows = len(mols) // molsPerRow
    if len(mols) % molsPerRow:
        nRows += 1

    if NEW_RENDER_ENGINE:
        if kekulize:
            _apply(mols, _kekulize)

        highlightBondLists = []
        for mol, highlightAtomList in zip(mols, highlightAtomLists):
            highlightBondList = []
            for bnd in mol.GetBonds():
                if bnd.GetBeginAtomIdx() in highlightAtomList and bnd.GetEndAtomIdx() in highlightAtomList:
                    highlightBondList.append(bnd.GetIdx())
            highlightBondLists.append(highlightBondList)

        panelx = size
        panely = size
        canvasx = panelx * molsPerRow
        canvasy = panely * nRows
        drawer = rdMolDraw2D.MolDraw2DCairo(canvasx, canvasy, panelx, panely)
        drawer.DrawMolecules(mols, highlightAtoms=highlightAtomLists, highlightBonds=highlightBondLists, legends=legends)
        drawer.FinishDrawing()
        f.write(drawer.GetDrawingText())

    else:
        image = draw.MolsToGridImage(mols, molsPerRow=molsPerRow, subImgSize=(size, size), legends=legends,
                                     highlightAtomLists=highlightAtomLists)
        image.save(f, format)

# ----------------------------------------------------------------------------------------------------------------------


def _mols2imageString(mols, size, legend, format, atomMapNumber=False, computeCoords=False, highlightAtomLists=None,
                      kekulize=True):
    if not mols:
        return ''
    if computeCoords:
        _apply(mols, _computeCoords, True)
    if atomMapNumber:
        _apply(mols, _atomMapNumber)
    image_data = StringIO.StringIO()
    _mols2imageStream(mols, image_data, format, size, legend, highlightAtomLists, kekulize)
    return image_data.getvalue()

# ----------------------------------------------------------------------------------------------------------------------


def _ctab2image(data, size, legend, sanitize=True, removeHs=True, strictParsing=True, atomMapNumber=False,
                computeCoords=False, kekulize=True):
    return _mols2imageString(_parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing),
                             size, legend, 'PNG', atomMapNumber, computeCoords, kekulize=kekulize)

# ----------------------------------------------------------------------------------------------------------------------


def _smiles2image(data, size, legend, computeCoords=False, delimiter=' ', smilesColumn=0, nameColumn=1,
                  titleLine=True, sanitize=True, atomMapNumber=False, kekulize=True):
    return _mols2imageString(_parseSMILESData(data, computeCoords=computeCoords, delimiter=delimiter,
                                              smilesColumn=smilesColumn, nameColumn=nameColumn, titleLine=titleLine,
                                              sanitize=sanitize),
                             size, legend, 'PNG', atomMapNumber, computeCoords, kekulize=kekulize)

# ----------------------------------------------------------------------------------------------------------------------


def _highlightSmilesFragment(data, smarts, size, legend, computeCoords=False, delimiter=' ', smilesColumn=0, 
                             nameColumn=1, titleLine=True, sanitize=True, atomMapNumber=False, kekulize=True,
                             force=False):
    mols = _parseSMILESData(data, computeCoords=computeCoords, delimiter=delimiter,
                            smilesColumn=smilesColumn, nameColumn=nameColumn, titleLine=titleLine, sanitize=sanitize)
    matches = _getMatches(mols, smarts, force)
    return _mols2imageString(mols, size, legend, 'PNG', atomMapNumber, computeCoords, matches, kekulize=kekulize)

# ----------------------------------------------------------------------------------------------------------------------


def _highlightCtabFragment(data, smarts, size, legend, sanitize=True, removeHs=True, strictParsing=True,
                           atomMapNumber=False, computeCoords=False, kekulize=True, force=False):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    matches = _getMatches(mols, smarts, force)
    return _mols2imageString(mols, size, legend, 'PNG', atomMapNumber, computeCoords, matches, kekulize=kekulize)

# ----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker import app
from bottle import request, response
from chembl_beaker.beaker.core_apps.svgImages.impl import _ctab2svg, _smiles2svg, _inchi2svg
from chembl_beaker.beaker.core_apps.svgImages.impl import _highlightCtabFragmentSVG, _highlightSmilesFragmentSVG
from chembl_beaker.beaker.utils.io import _parseFlag
import base64

# ----------------------------------------------------------------------------------------------------------------------


def ctab2svgView(data, params):

    kwargs = dict()
    kwargs['size'] = int(params.get('size', 200))
    kwargs['legend'] = params.get('legend', '')
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['kekulize'] = _parseFlag(params.get('kekulize', True))
    kwargs['wedgeBonds'] = _parseFlag(params.get('wedgeBonds', True))
    kwargs['fitImage'] = _parseFlag(params.get('fitImage', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))

    response.content_type = 'image/svg+xml'
    return _ctab2svg(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2svg/<ctab>', method=['OPTIONS', 'GET'], name="ctab2svg")
def ctab2svg(ctab):
    """
Converts CTAB to SVG vector graphic. CTAB is urlsafe_base64 encoded string containing single molfile or
concatenation of multiple molfiles. Size is the optional size of image in pixels (default value is 200 px).
Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ctab2svg/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_") > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}ctab2svg/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?computeCoords=0 > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}ctab2svg/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?atomMapNumber=1 > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}ctab2svg/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}ctab2svg/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin.svg
    """

    data = base64.urlsafe_b64decode(ctab)
    return ctab2svgView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2svg', method=['OPTIONS', 'POST'], name="ctab2svg")
def ctab2svg():
    """
Converts CTAB to SVG vector graphic. CTAB is either single molfile or SDF file. Size is the optional size of
image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}ctab2svg > aspirin.svg
    curl -X POST -F "file=@aspirin.mol" -F "computeCoords=0" ${BEAKER_ROOT_URL}ctab2svg > aspirin.svg
    curl -X POST -F "file=@aspirin.mol" -F "atomMapNumber=1" ${BEAKER_ROOT_URL}ctab2svg > aspirin.svg
    curl -X POST -F "file=@aspirin.mol" -F "legend=aspirin" ${BEAKER_ROOT_URL}ctab2svg > aspirin.svg
    curl -X POST -F "file=@aspirin.mol" -F "size=400" ${BEAKER_ROOT_URL}ctab2svg > aspirin.svg
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return ctab2svgView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def smiles2svgView(data, params):

    kwargs = dict()
    kwargs['legend'] = params.get('legend','')
    kwargs['size'] = int(params.get('size', 200))
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['kekulize'] = _parseFlag(params.get('kekulize', True))
    kwargs['wedgeBonds'] = _parseFlag(params.get('wedgeBonds', True))
    kwargs['fitImage'] = _parseFlag(params.get('fitImage', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    response.content_type = 'image/svg+xml'
    return _smiles2svg(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2svg/<smiles>', method=['OPTIONS', 'GET'], name="smiles2svg")
def smiles2svg(smiles):
    """
Converts SMILES to SVG vector graphic. This method accepts urlsafe_base64 encoded string containing single or
multiple SMILES optionally containing header line, specific to *.smi format. Size is the optional size of image in
pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}smiles2svg/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_") > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}smiles2svg/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?atomMapNumber=1 > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}smiles2svg/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}smiles2svg/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}smiles2svg/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_") > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}smiles2svg/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}smiles2svg/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin.svg
    """

    data = base64.urlsafe_b64decode(smiles)
    return smiles2svgView(data, params=request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2svg', method=['OPTIONS', 'POST'], name="smiles2svg")
def smiles2svg():
    """
Converts SMILES to SVG vector graphic. This method accepts single or multiple SMILES or *.smi file. Size is the
optional size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST --data-binary @aspirin_no_header.smi ${BEAKER_ROOT_URL}smiles2svg > aspirin.svg
    curl -X POST -F "file=@aspirin_no_header.smi" -F "atomMapNumber=1" ${BEAKER_ROOT_URL}smiles2svg > aspirin.svg
    curl -X POST -F "file=@aspirin_no_header.smi" -F "legend=aspirin" ${BEAKER_ROOT_URL}smiles2svg > aspirin.svg
    curl -X POST -F "file=@aspirin_no_header.smi" -F "size=400" ${BEAKER_ROOT_URL}smiles2svg > aspirin.svg
    curl -X POST --data-binary @aspirin_with_header.smi ${BEAKER_ROOT_URL}smiles2svg > aspirin.svg
    curl -X POST -F "file=@aspirin_with_header.smi" -F "legend=aspirin" ${BEAKER_ROOT_URL}smiles2svg > aspirin.svg
    curl -X POST -F "file=@aspirin_with_header.smi" -F "size=400" ${BEAKER_ROOT_URL}smiles2svg > aspirin.svg
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return smiles2svgView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def highlightSmilesFragmentSvgView(data, params):

    kwargs = dict()
    smarts = params.get('smarts', '')
    kwargs['legend'] = params.get('legend', '')
    kwargs['size'] = int(params.get('size', 200))
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['kekulize'] = _parseFlag(params.get('kekulize', True))
    kwargs['wedgeBonds'] = _parseFlag(params.get('wedgeBonds', True))
    kwargs['fitImage'] = _parseFlag(params.get('fitImage', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))
    kwargs['force'] = _parseFlag(params.get('force', True))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    response.content_type = 'image/svg+xml'
    return _highlightSmilesFragmentSVG(data, smarts, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------

@app.route('/highlightSmilesFragmentSvg/<smarts>/<smiles>', method=['OPTIONS', 'GET'], name="highlightSmilesFragmentSvg")
def highlightSmilesFragmentSvg(smarts, smiles):
    """
Converts SMILES to SVG vector graphic with a highlighted fragment described as SMARTS. 
This method accepts urlsafe_base64 encoded string containing SMARTS with a fragment to be highlighted, urlsafe_base64 
encoded string containing single or multiple SMILES optionally containing header line, specific to *.smi format. 
Size is the optional size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_") > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?atomMapNumber=1 > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_") > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg/$(cat aspirin.sma | base64 -w 0 | tr "+/" "-_")/$(cat CHEMBL1999443.smi | base64 -w 0 | tr "+/" "-_")?force=true > out_highlighted_forced.svg
    """

    data = base64.urlsafe_b64decode(smiles)

    params = request.params
    params['smarts'] = base64.urlsafe_b64decode(smarts)

    return highlightSmilesFragmentSvgView(data, params=params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/highlightSmilesFragmentSvg', method=['OPTIONS', 'POST'], name="highlightSmilesFragmentSvg")
def highlightSmilesFragmentSvg():
    """
Converts SMILES to SVG vector graphic with a highlighted fragment described as SMARTS. 
This method accepts SMARTS and single or multiple SMILES or *.smi file. Size is the
optional size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST -F "file=@aspirin_no_header.smi" -F "smarts=c1ccccc1" ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin_no_header.smi" -F "smarts=c1ccccc1" -F "atomMapNumber=1" ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin_no_header.smi" -F "smarts=c1ccccc1" -F "legend=aspirin" ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin_no_header.smi" -F "smarts=c1ccccc1" -F "size=400" ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin_with_header.smi" -F "smarts=c1ccccc1" -F "legend=aspirin" ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin_with_header.smi" -F "smarts=c1ccccc1" -F "size=400" ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@CHEMBL1999443.smi" -F "smarts=[#6]1:[#6]:[#6]:[#6]:[#6](:[#6]:1-[#6](-[#8])=[#8])-[#8]-[#6](-[#6])=[#8]" -F "force=true" ${BEAKER_ROOT_URL}highlightSmilesFragmentSvg > out_highlighted_forced.svg
    curl -X POST -F "file=@CHEMBL1999443.smi" -F "smarts=@aspirin.sma" -F "force=true" ${BEAKER_ROOT_URL}highlightSmilesFragment > highlightSmilesFragmentSvg.svg
    """

    number_of_files = len(request.files)
    data = None
    if number_of_files:
        if number_of_files == 1:
            data = request.files.values()[0].file.read()
        elif number_of_files == 2:
            data = request.files['file'].file.read()
            smarts = request.files['smarts'].file.read()
            request.params['smarts'] = smarts
    else:
        data = request.body.read()
    return highlightSmilesFragmentSvgView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def highlightCtabFragmentSvgView(data, params):

    kwargs = dict()
    smarts = params.get('smarts', '')
    kwargs['size'] = int(params.get('size', 200))
    kwargs['legend'] = params.get('legend', '')
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['kekulize'] = _parseFlag(params.get('kekulize', True))
    kwargs['wedgeBonds'] = _parseFlag(params.get('wedgeBonds', True))
    kwargs['fitImage'] = _parseFlag(params.get('fitImage', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))
    kwargs['force'] = _parseFlag(params.get('force', True))

    response.content_type = 'image/svg+xml'
    return _highlightCtabFragmentSVG(data, smarts, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/highlightCtabFragmentSvg/<smarts>/<ctab>', method=['OPTIONS', 'GET'], name="highlightCtabFragmentSvg")
def highlightCtabFragmentSvg(smarts, ctab):
    """
Converts SMILES to SVG vector graphic with a highlighted fragment described as SMARTS. 
SMARTS is urlsafe_base64 encoded string containing fragment ot ge highlighted.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles. 
Size is the optional size of image in pixels (default value is 200 px).
Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_") > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?computeCoords=0 > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?atomMapNumber=1 > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragmentSvg/$(echo c1ccccc1 | base64 -w 0 | tr "+/" "-_")/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin_highlighted.svg
    curl -X GET ${BEAKER_ROOT_URL}highlightCtabFragmentSvg/$(cat aspirin.sma | base64 -w 0 | tr "+/" "-_")/$(cat CHEMBL1999443.mol | base64 -w 0 | tr "+/" "-_")?force=true > out_highlighted_forced.svg
    """

    data = base64.urlsafe_b64decode(ctab)

    params = request.params
    params['smarts'] = base64.urlsafe_b64decode(smarts)

    return highlightCtabFragmentSvgView(data, params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/highlightCtabFragmentSvg', method=['OPTIONS', 'POST'], name="highlightCtabFragmentSvg")
def highlightCtabFragmentSvg():
    """
Converts SMILES to SVG vector graphic with a highlighted fragment described as SMARTS.
SMARTS describes the fragment to be highlighted. 
CTAB is either single molfile or SDF file. Size is the optional size of
image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" ${BEAKER_ROOT_URL}highlightCtabFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" -F "computeCoords=0" ${BEAKER_ROOT_URL}highlightCtabFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" -F "atomMapNumber=1" ${BEAKER_ROOT_URL}highlightCtabFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" -F "legend=aspirin" ${BEAKER_ROOT_URL}highlightCtabFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@aspirin.mol" -F "smarts=c1ccccc1" -F "size=400" ${BEAKER_ROOT_URL}highlightCtabFragmentSvg > aspirin_highlighted.svg
    curl -X POST -F "file=@CHEMBL1999443.mol" -F "smarts=[#6]1:[#6]:[#6]:[#6]:[#6](:[#6]:1-[#6](-[#8])=[#8])-[#8]-[#6](-[#6])=[#8]" -F "force=true" ${BEAKER_ROOT_URL}highlightCtabFragmentSvg > out_highlighted_forced.svg
    curl -X POST -F "file=@CHEMBL1999443.mol" -F "smarts=@aspirin.sma" -F "force=true" ${BEAKER_ROOT_URL}highlightCtabFragmentSvg > out_highlighted_forced.svg
    """

    number_of_files = len(request.files)
    data = None
    if number_of_files:
        if number_of_files == 1:
            data = request.files.values()[0].file.read()
        elif number_of_files == 2:
            data = request.files['file'].file.read()
            smarts = request.files['smarts'].file.read()
            request.params['smarts'] = smarts
    else:
        data = request.body.read()
    return highlightCtabFragmentSvgView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

def inchi2svgView(data, params):

    kwargs = dict()
    kwargs['size'] = int(params.get('size', 200))
    kwargs['legend'] = params.get('legend','')
    kwargs['kekulize'] = _parseFlag(params.get('kekulize', True))
    kwargs['wedgeBonds'] = _parseFlag(params.get('wedgeBonds', True))
    kwargs['fitImage'] = _parseFlag(params.get('fitImage', True))
    kwargs['atomMapNumber'] = _parseFlag(params.get('atomMapNumber', False))
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))

    response.content_type = 'image/svg+xml'
    return _inchi2svg(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/inchi2svg/<inchi>', method=['OPTIONS', 'GET'], name="inchi2svg")
def inchi2svg(inchi):
    """
Converts InChI to SVG vector graphic. This method accepts urlsafe_base64 encoded string containing single or
multiple InChIs. Size is the optional size of image in pixels (default value is 200 px).
Legend is optional label in the bottom of image.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}inchi2svg/$(cat aspirin.inchi | base64 -w 0 | tr "+/" "-_") > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}inchi2svg/$(cat aspirin.inchi | base64 -w 0 | tr "+/" "-_")?legend=aspirin > aspirin.svg
    curl -X GET ${BEAKER_ROOT_URL}inchi2svg/$(cat aspirin.inchi | base64 -w 0 | tr "+/" "-_")?size=400 > aspirin.svg
    """

    inchis = base64.urlsafe_b64decode(inchi)
    return inchi2svgView(inchis, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/inchi2svg', method=['OPTIONS', 'POST'], name="inchi2svg")
def inchi2svg():
    """
Converts InChI to SVG vector graphic. This method accepts single or multiple InChIs. Size is the
optional size of image in pixels (default value is 200 px). Legend is optional label in the bottom of image.
cURL examples:

    curl -X POST --data-binary @aspirin.inchi ${BEAKER_ROOT_URL}inchi2svg > aspirin.svg
    curl -X POST -F "file=@aspirin.inchi" -F "legend=aspirin" ${BEAKER_ROOT_URL}inchi2svg > aspirin.svg
    curl -X POST -F "file=@aspirin.inchi" -F "size=400" ${BEAKER_ROOT_URL}inchi2svg > aspirin.svg
    """

    inchis = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return inchi2svgView(inchis, request.params)

# ----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from chembl_beaker.beaker import app
from bottle import request
from chembl_beaker.beaker.core_apps.conversions.impl import _ctab2smiles, _smiles2ctab, _inchi2ctab, _ctab2smarts
from chembl_beaker.beaker.core_apps.conversions.impl import _ctab2inchi, _inchi2inchiKey, _ctab2xyz
from chembl_beaker.beaker.core_apps.conversions.impl import _canonicalize_smiles, _ctab2inchiKey
from chembl_beaker.beaker.core_apps.conversions.impl import _smiles2inchi, _smiles2inchiKey
from chembl_beaker.beaker.core_apps.conversions.impl import _smarts2ctab
from chembl_beaker.beaker.utils.io import _parseFlag
import base64

# ----------------------------------------------------------------------------------------------------------------------


def ctab2smilesView(data, params):
    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['nameHeader'] = params.get('nameHeader', 'Name')
    kwargs['includeHeader'] = _parseFlag(params.get('includeHeader', True))
    kwargs['isomericSmiles'] = _parseFlag(params.get('isomericSmiles', False))
    kwargs['kekuleSmiles'] = _parseFlag(params.get('kekuleSmiles', False))
    return _ctab2smiles(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2smiles/<ctab>', method=['OPTIONS', 'GET'], name="ctab2smiles")
def ctab2smiles(ctab):
    """
Converts CTAB to SMILES format. CTAB is urlsafe_base64 encoded string containing single molfile or concatenation
of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ctab2smiles/$(cat isomeric.mol | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}ctab2smiles/$(cat isomeric.mol | base64 -w 0 | tr "+/" "-_")?isomericSmiles=1
    curl -X GET "${BEAKER_ROOT_URL}ctab2smiles/"$(cat non_kekule.mol | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=0&sanitize=1"
    curl -X GET "${BEAKER_ROOT_URL}ctab2smiles/"$(cat non_kekule.mol | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=0&sanitize=0"
    curl -X GET "${BEAKER_ROOT_URL}ctab2smiles/"$(cat non_kekule.mol | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=1&sanitize=1"
    curl -X GET "${BEAKER_ROOT_URL}ctab2smiles/"$(cat explicitHs.mol | base64 -w 0 | tr "+/" "-_")"?removeHs=0"
    """

    data = base64.urlsafe_b64decode(ctab)
    return ctab2smilesView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2smiles', method=['OPTIONS', 'POST'], name="ctab2smiles")
def ctab2smiles():
    """
Converts CTAB to SMILES format. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST -F "file=@isomeric.mol" ${BEAKER_ROOT_URL}ctab2smiles
    curl -X POST -F "file=@isomeric.mol" -F "isomericSmiles=1" ${BEAKER_ROOT_URL}ctab2smiles
    curl -X POST -F "file=@non_kekule.mol" -F "kekuleSmiles=0" -F "sanitize=1" ${BEAKER_ROOT_URL}ctab2smiles
    curl -X POST -F "file=@non_kekule.mol" -F "kekuleSmiles=0" -F "sanitize=0" ${BEAKER_ROOT_URL}ctab2smiles
    curl -X POST -F "file=@non_kekule.mol" -F "kekuleSmiles=1" -F "sanitize=1" ${BEAKER_ROOT_URL}ctab2smiles
    curl -X POST -F "file=@explicitHs.mol" -F "removeHs=0" ${BEAKER_ROOT_URL}ctab2smiles
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return ctab2smilesView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def ctab2smartsView(data, params):
    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['isomericSmiles'] = _parseFlag(params.get('isomericSmiles', False))
    return _ctab2smarts(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2smarts/<ctab>', method=['OPTIONS', 'GET'], name="ctab2smarts")
def ctab2smarts(ctab):
    """
Converts CTAB to SMARTS format. CTAB is urlsafe_base64 encoded string containing single molfile or concatenation
of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ctab2smarts/$(cat isomeric.mol | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}ctab2smarts/$(cat isomeric.mol | base64 -w 0 | tr "+/" "-_")?isomericSmiles=1
    curl -X GET "${BEAKER_ROOT_URL}ctab2smarts/"$(cat non_kekule.mol | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=0&sanitize=1"
    curl -X GET "${BEAKER_ROOT_URL}ctab2smarts/"$(cat non_kekule.mol | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=0&sanitize=0"
    curl -X GET "${BEAKER_ROOT_URL}ctab2smarts/"$(cat non_kekule.mol | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=1&sanitize=1"
    curl -X GET "${BEAKER_ROOT_URL}ctab2smarts/"$(cat explicitHs.mol | base64 -w 0 | tr "+/" "-_")"?removeHs=0"
    """

    data = base64.urlsafe_b64decode(ctab)
    return ctab2smartsView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2smarts', method=['OPTIONS', 'POST'], name="ctab2smarts")
def ctab2smarts():
    """
Converts CTAB to SMILES format. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST -F "file=@isomeric.mol" ${BEAKER_ROOT_URL}ctab2smiles
    curl -X POST -F "file=@isomeric.mol" -F "isomericSmiles=1" ${BEAKER_ROOT_URL}ctab2smarts
    curl -X POST -F "file=@non_kekule.mol" -F "kekuleSmiles=0" -F "sanitize=1" ${BEAKER_ROOT_URL}ctab2smarts
    curl -X POST -F "file=@non_kekule.mol" -F "kekuleSmiles=0" -F "sanitize=0" ${BEAKER_ROOT_URL}ctab2smarts
    curl -X POST -F "file=@non_kekule.mol" -F "kekuleSmiles=1" -F "sanitize=1" ${BEAKER_ROOT_URL}ctab2smarts
    curl -X POST -F "file=@explicitHs.mol" -F "removeHs=0" ${BEAKER_ROOT_URL}ctab2smarts
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return ctab2smartsView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def ctab2xyzView(data, params):
    computeCoords = _parseFlag(params.get('computeCoords', True))
    return _ctab2xyz(data, computeCoords)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2xyz/<ctab>', method=['OPTIONS', 'GET'], name="ctab2xyz")
def ctab2xyz(ctab):
    """
Converts the molecules in the CTAB to xyz format. CTAB is a single molfile
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ctab2xyz/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    curl -X GET "${BEAKER_ROOT_URL}ctab2xyz/"$(cat mcs.sdf | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}ctab2xyz/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")"?computeCoords=0"
    """
    data = base64.urlsafe_b64decode(ctab)
    return ctab2xyzView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2xyz', method=['OPTIONS', 'POST'], name="ctab2xyz")
def ctab2xyz():
    """
Converts the molecules in the CTAB to xyz format. CTAB is a single molfile
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}ctab2xyz
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}ctab2xyz
    curl -X POST -F "file=@mcs.sdf" ${BEAKER_ROOT_URL}ctab2xyz
    curl -X POST -F "file=@aspirin.mol" -F "computeCoords=0"  ${BEAKER_ROOT_URL}ctab2xyz
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return ctab2xyzView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def smiles2ctabView(data, params):
    kwargs = dict()
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    return _smiles2ctab(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2ctab/<smiles>', method=['OPTIONS', 'GET'], name="smiles2ctab")
def smiles2ctab(smiles):
    """
Converts SMILES to CTAB. This method accepts urlsafe_base64 encoded string containing single or multiple SMILES
optionally containing header line, specific to *.smi format.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}smiles2ctab/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles2ctab/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET "${BEAKER_ROOT_URL}smiles2ctab/"$(cat rules.smi | base64 -w 0 | tr "+/" "-_")"?computeCoords=0"
    curl -X GET ${BEAKER_ROOT_URL}smiles2ctab/$(cat mcs.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles2ctab/$(cat mcs_no_header.smi | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(smiles)
    return smiles2ctabView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2ctab', method=['OPTIONS', 'POST'], name="smiles2ctab")
def smiles2ctab():
    """
Converts SMILES to CTAB. This method accepts single or multiple SMILES or *.smi file.
cURL examples:

    curl -X POST -F "file=@aspirin_with_header.smi" ${BEAKER_ROOT_URL}smiles2ctab
    curl -X POST -F "file=@aspirin_no_header.smi" ${BEAKER_ROOT_URL}smiles2ctab
    curl -X POST -F "file=@rules.smi" -F "computeCoords=0"  ${BEAKER_ROOT_URL}smiles2ctab
    curl -X POST -F "file=@mcs.smi" ${BEAKER_ROOT_URL}smiles2ctab
    curl -X POST -F "file=@mcs_no_header.smi" ${BEAKER_ROOT_URL}smiles2ctab
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return smiles2ctabView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def smarts2ctabView(data, params):
    kwargs = dict()
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', True))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))

    return _smarts2ctab(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smarts2ctab/<smarts>', method=['OPTIONS', 'GET'], name="smarts2ctab")
def smarts2ctab(smarts):
    """
Converts SMARTS to CTAB. This method accepts urlsafe_base64 encoded string containing single or multiple SMARTS.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}smarts2ctab/$(echo '[CH2X4][CX3](=[OX1])[NX3H2]' | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smarts2ctab/$(cat amino.sma | base64 -w 0 | tr "+/" "-_")
    curl -X GET "${BEAKER_ROOT_URL}smarts2ctab/"$(cat amino.sma | base64 -w 0 | tr "+/" "-_")"?computeCoords=0"
    """

    data = base64.urlsafe_b64decode(smarts)
    return smarts2ctabView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smarts2ctab', method=['OPTIONS', 'POST'], name="smarts2ctab")
def smarts2ctab():
    """
Converts SMARTS to CTAB. This method accepts single or multiple SMARTS.
cURL examples:

    curl -X POST -F "file=@amino.sma" ${BEAKER_ROOT_URL}smarts2ctab
    curl -X POST -F "file=@amino.sma" -F "computeCoords=0"  ${BEAKER_ROOT_URL}smarts2ctab
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return smarts2ctabView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def smiles2inchiView(data, params):
    kwargs = dict()
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', False))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    return _smiles2inchi(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2inchi/<smiles>', method=['OPTIONS', 'GET'], name="smiles2inchi")
def smiles2inchi(smiles):
    """
Converts SMILES to InChi. This method accepts urlsafe_base64 encoded string containing single or multiple SMILES
optionally containing header line, specific to *.smi format.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}smiles2inchi/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles2inchi/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles2inchi/$(cat mcs.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles2inchi/$(cat mcs_no_header.smi | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(smiles)
    return smiles2inchiView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2inchi', method=['OPTIONS', 'POST'], name="smiles2inchi")
def smiles2inchi():
    """
Converts SMILES to InChi. This method accepts single or multiple SMILES or *.smi file.
cURL examples:

    curl -X POST -F "file=@aspirin_with_header.smi" ${BEAKER_ROOT_URL}smiles2inchi
    curl -X POST -F "file=@aspirin_no_header.smi" ${BEAKER_ROOT_URL}smiles2inchi
    curl -X POST -F "file=@mcs.smi" ${BEAKER_ROOT_URL}smiles2inchi
    curl -X POST -F "file=@mcs_no_header.smi" ${BEAKER_ROOT_URL}smiles2inchi
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return smiles2inchiView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def smiles2inchiKeyView(data, params):
    kwargs = dict()
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', False))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    return _smiles2inchiKey(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2inchiKey/<smiles>', method=['OPTIONS', 'GET'], name="smiles2inchiKey")
def smiles2inchiKey(smiles):
    """
Converts SMILES to InChi Key. This method accepts urlsafe_base64 encoded string containing single or multiple SMILES
optionally containing header line, specific to *.smi format.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}smiles2inchiKey/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles2inchiKey/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles2inchiKey/$(cat mcs.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles2inchiKey/$(cat mcs_no_header.smi | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(smiles)
    return smiles2inchiKeyView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles2inchiKey', method=['OPTIONS', 'POST'], name="smiles2inchiKey")
def smiles2inchiKey():
    """
Converts SMILES to InChi Key. This method accepts single or multiple SMILES or *.smi file.
cURL examples:

    curl -X POST -F "file=@aspirin_with_header.smi" ${BEAKER_ROOT_URL}smiles2inchiKey
    curl -X POST -F "file=@aspirin_no_header.smi" ${BEAKER_ROOT_URL}smiles2inchi
    curl -X POST -F "file=@mcs.smi" ${BEAKER_ROOT_URL}smiles2inchiKey
    curl -X POST -F "file=@mcs_no_header.smi" ${BEAKER_ROOT_URL}smiles2inchiKey
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return smiles2inchiKeyView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def canonicalizeSmilesView(data, params):
    kwargs = dict()
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', False))
    kwargs['in_delimiter'] = params.get('in_delimiter', ' ')
    kwargs['out_delimiter'] = params.get('out_delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['nameHeader'] = params.get('nameHeader', 'Name')
    kwargs['includeHeader'] = _parseFlag(params.get('includeHeader', True))
    kwargs['isomericSmiles'] = _parseFlag(params.get('isomericSmiles', False))
    kwargs['kekuleSmiles'] = _parseFlag(params.get('kekuleSmiles', False))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    return _canonicalize_smiles(data, **kwargs)
# ----------------------------------------------------------------------------------------------------------------------


@app.route('/canonicalizeSmiles/<smiles>', method=['OPTIONS', 'GET'], name="canonicalizeSmiles")
def canonicalizeSmiles(smiles):
    """
Converts SMILES to canonical SMILES. This method accepts urlsafe_base64 encoded string containing single or multiple
SMILES optionally containing header line, specific to *.smi format.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}canonicalizeSmiles/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}canonicalizeSmiles/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET "${BEAKER_ROOT_URL}canonicalizeSmiles/"$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")"?out_delimiter=|&nameHeader=foo"
    curl -X GET "${BEAKER_ROOT_URL}canonicalizeSmiles/"$(cat non_kekule.smi | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=0&sanitize=0"
    curl -X GET "${BEAKER_ROOT_URL}canonicalizeSmiles/"$(cat non_kekule.smi | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=0&sanitize=1"
    curl -X GET "${BEAKER_ROOT_URL}canonicalizeSmiles/"$(cat non_kekule.smi | base64 -w 0 | tr "+/" "-_")"?kekuleSmiles=1&sanitize=1"
    curl -X GET "${BEAKER_ROOT_URL}canonicalizeSmiles/"$(cat isomeric.smi | base64 -w 0 | tr "+/" "-_")"?isomericSmiles=1"
    """

    data = base64.urlsafe_b64decode(smiles)
    return canonicalizeSmilesView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/canonicalizeSmiles', method=['OPTIONS', 'POST'], name="canonicalizeSmiles")
def canonicalizeSmiles():
    """
Converts SMILES to canonical SMILES. This method accepts single or multiple SMILES or *.smi file.
cURL examples:

    curl -X POST --data-binary @aspirin_no_header.smi ${BEAKER_ROOT_URL}canonicalizeSmiles
    curl -X POST --data-binary @aspirin_with_header.smi ${BEAKER_ROOT_URL}canonicalizeSmiles
    curl -X POST -F "file=@aspirin_with_header.smi" -F "out_delimiter=|" -F "nameHeader=foo" ${BEAKER_ROOT_URL}canonicalizeSmiles
    curl -X POST -F "file=@non_kekule.smi" -F "kekuleSmiles=0" -F "sanitize=0" ${BEAKER_ROOT_URL}canonicalizeSmiles
    curl -X POST -F "file=@non_kekule.smi" -F "kekuleSmiles=0" -F "sanitize=1" ${BEAKER_ROOT_URL}canonicalizeSmiles
    curl -X POST -F "file=@non_kekule.smi" -F "kekuleSmiles=1" -F "sanitize=1" ${BEAKER_ROOT_URL}canonicalizeSmiles
    curl -X POST -F "file=@isomeric.smi" ${BEAKER_ROOT_URL}canonicalizeSmiles
    curl -X POST -F "file=@isomeric.smi" -F "isomericSmiles=1" ${BEAKER_ROOT_URL}canonicalizeSmiles
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return canonicalizeSmilesView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/inchi2ctab/<inchi>', method=['OPTIONS', 'GET'], name="inchi2ctab")
def inchi2ctab(inchi):
    """
Converts InChi to CTAB. This method accepts urlsafe_base64 encoded string containing one or multiple InChis.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}inchi2ctab/$(cat aspirin.inchi | base64 -w 0 | tr "+/" "-_")
    """

    inchis = base64.urlsafe_b64decode(inchi)
    return _inchi2ctab(inchis)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/inchi2ctab', method=['OPTIONS', 'POST'], name="inchi2ctab")
def inchi2ctab():
    """
Converts InChi to CTAB. This method accepts one or multiple InChis.
cURL examples:

    curl -X POST --data-binary @aspirin.inchi ${BEAKER_ROOT_URL}inchi2ctab
    curl -X POST -F "file=@aspirin.inchi" ${BEAKER_ROOT_URL}inchi2ctab
    """

    inchis = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return _inchi2ctab(inchis)

# ----------------------------------------------------------------------------------------------------------------------


def ctab2inchiView(data, params):
    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    return _ctab2inchi(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2inchi/<ctab>', method=['OPTIONS', 'GET'], name="ctab2inchi")
def ctab2inchi(ctab):
    """
Converts CTAB to InChis. CTAB is urlsafe_base64 encoded string containing single molfile or concatenation
of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ctab2inchi/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return ctab2inchiView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2inchi', method=['OPTIONS', 'POST'], name="ctab2inchi")
def ctab2inchi():
    """
Converts CTAB to InChis. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}ctab2inchi
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}ctab2inchi
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return ctab2inchiView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def ctab2inchiKeyView(data, params):
    kwargs = dict()
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    return _ctab2inchiKey(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2inchiKey/<ctab>', method=['OPTIONS', 'GET'], name="ctab2inchiKey")
def ctab2inchiKey(ctab):
    """
Converts CTAB to InChi Keys. CTAB is urlsafe_base64 encoded string containing single molfile or concatenation
of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ctab2inchiKey/$(cat aspirin.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return ctab2inchiKeyView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab2inchiKey', method=['OPTIONS', 'POST'], name="ctab2inchiKey")
def ctab2inchiKey():
    """
Converts CTAB to InChi Keys. CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin.mol ${BEAKER_ROOT_URL}ctab2inchiKey
    curl -X POST -F "file=@aspirin.mol" ${BEAKER_ROOT_URL}ctab2inchiKey
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return ctab2inchiKeyView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/inchi2inchiKey/<inchi>', method=['OPTIONS', 'GET'], name="inchi2inchiKey")
def inchi2inchiKey(inchi):
    """
Converts InChis to InChiKeys. This method accepts urlsafe_base64 encoded string containing one or multiple InChis.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}inchi2inchiKey/$(cat aspirin.inchi | base64 -w 0 | tr "+/" "-_")
    """

    inchis = base64.urlsafe_b64decode(inchi)
    return _inchi2inchiKey(inchis)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/inchi2inchiKey', method=['OPTIONS', 'POST'], name="inchi2inchiKey")
def inchi2inchiKey():
    """
Converts InChis to InChiKeys. This method accepts one or multiple InChis.
cURL examples:

    curl -X POST --data-binary @aspirin.inchi ${BEAKER_ROOT_URL}inchi2inchiKey
    curl -X POST -F "file=@aspirin.inchi" ${BEAKER_ROOT_URL}inchi2inchiKey
    """

    inchis = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return _inchi2inchiKey(inchis)

# ----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

# ----------------------------------------------------------------------------------------------------------------------

from rdkit.Chem import AllChem
from chembl_beaker.beaker import app
from bottle import request
from chembl_beaker.beaker.core_apps.D3Coords.impl import _ctab23D, _smiles23D
from chembl_beaker.beaker.utils.io import _parseFlag
import base64

# ----------------------------------------------------------------------------------------------------------------------


def ctab23DView(data, params):
    kwargs = dict()
    kwargs['multi'] = int(params.get('multi', False))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
    kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
    kwargs['mmff'] = _parseFlag(params.get('mmff', False))

    return _ctab23D(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab23D/<ctab>', method=['OPTIONS', 'GET'], name="ctab23D")
def ctab23D(ctab):
    """
Generate 3D coordinates from molfile using Universal Force Field.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cuRL examples:

    curl -X GET ${BEAKER_ROOT_URL}ctab23D/$(cat no_coords.mol | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return ctab23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/ctab23D', method=['OPTIONS', 'POST'], name="ctab23D")
def ctab23D():
    """
Generate 3D coordinates from molfile using Universal Force Field.
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @no_coords.mol ${BEAKER_ROOT_URL}ctab23D
    curl -X POST -F "file=@no_coords.mol" ${BEAKER_ROOT_URL}ctab23D
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return ctab23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


def smiles23DView(data, params):
    kwargs = dict()
    kwargs['multi'] = int(params.get('multi', False))
    kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', False))
    kwargs['delimiter'] = params.get('delimiter', ' ')
    kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
    kwargs['nameColumn'] = int(params.get('nameColumn', 1))
    kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
    kwargs['mmff'] = _parseFlag(params.get('mmff', False))

    if params.get('titleLine') is None and not data.startswith('SMILES Name'):
        kwargs['titleLine'] = False
    else:
        kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

    return _smiles23D(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles23D/<ctab>', method=['OPTIONS', 'GET'], name="smiles23D")
def smiles23D(ctab):
    """
Generate 3D coordinates from SMILES using Universal Force Field.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}smiles23D/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}smiles23D/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")
    """

    data = base64.urlsafe_b64decode(ctab)
    return smiles23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------


@app.route('/smiles23D', method=['OPTIONS', 'POST'], name="smiles23D")
def smiles23D():
    """
Generate 3D coordinates from SMILES using Universal Force Field.
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin_with_header.smi ${BEAKER_ROOT_URL}smiles23D
    curl -X POST -F "file=@aspirin_with_header.smi" ${BEAKER_ROOT_URL}smiles23D
    curl -X POST --data-binary @aspirin_no_header.smi ${BEAKER_ROOT_URL}smiles23D
    curl -X POST -F "file=@aspirin_no_header.smi" ${BEAKER_ROOT_URL}smiles23D
    """

    data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
    return smiles23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

if hasattr(AllChem, 'MMFFOptimizeMolecule'):

    # TODO: this one is redundant (but you have to remember about conditional availability)
    def MMFFctab23DView(data, params):
        kwargs = dict()
        kwargs['multi'] = int(params.get('multi', False))
        kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
        kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
        kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
        kwargs['mmff'] = True

        return _ctab23D(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/MMFFctab23D/<ctab>', method=['OPTIONS', 'GET'], name="MMFFctab23D")
    def MMFFctab23D(ctab):
        """
Generate 3D coordinates from molfile using Merck Molecular Force Field.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}MMFFctab23D/$(cat no_coords.mol | base64 -w 0 | tr "+/" "-_")
        """

        data = base64.urlsafe_b64decode(ctab)
        return MMFFctab23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/MMFFctab23D', method=['OPTIONS', 'POST'], name="MMFFctab23D")
    def MMFFctab23D():
        """
Generate 3D coordinates from molfile using Merck Molecular Force Field.
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @no_coords.mol ${BEAKER_ROOT_URL}MMFFctab23D
    curl -X POST -F "file=@no_coords.mol" ${BEAKER_ROOT_URL}MMFFctab23D    
        """

        data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
        return MMFFctab23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    def MMFFsmiles23DView(data, params):
        kwargs = dict()
        kwargs['multi'] = int(params.get('multi', False))
        kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', False))
        kwargs['delimiter'] = params.get('delimiter', ' ')
        kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
        kwargs['nameColumn'] = int(params.get('nameColumn', 1))
        kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
        kwargs['mmff'] = True

        if params.get('titleLine') is None and not data.startswith('SMILES Name'):
            kwargs['titleLine'] = False
        else:
            kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

        return _smiles23D(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/MMFFsmiles23D/<ctab>', method=['OPTIONS', 'GET'], name="MMFFsmiles23D")
    def MMFFsmiles23D(ctab):
        """
Generate 3D coordinates from SMILES using Merck Molecular Force Field.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}MMFFsmiles23D/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}MMFFsmiles23D/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")    
        """

        data = base64.urlsafe_b64decode(ctab)
        return MMFFsmiles23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/MMFFsmiles23D', method=['OPTIONS', 'POST'], name="MMFFsmiles23D")
    def MMFFsmiles23D():
        """
Generate 3D coordinates from SMILES using Merck Molecular Force Field.
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin_with_header.smi ${BEAKER_ROOT_URL}MMFFsmiles23D
    curl -X POST -F "file=@aspirin_with_header.smi" ${BEAKER_ROOT_URL}MMFFsmiles23D
    curl -X POST --data-binary @aspirin_no_header.smi ${BEAKER_ROOT_URL}MMFFsmiles23D
    curl -X POST -F "file=@aspirin_no_header.smi" ${BEAKER_ROOT_URL}MMFFsmiles23D
        """

        data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
        return MMFFsmiles23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

if hasattr(AllChem, 'ETKDG'):

    # TODO: this one is redundant (but you have to remember about conditional availability)
    def ETKDGctab23DView(data, params):
        kwargs = dict()
        kwargs['multi'] = int(params.get('multi', False))
        kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
        kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
        kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
        kwargs['mmff'] = True

        return _ctab23D(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('ETKDGctab23D/<ctab>', method=['OPTIONS', 'GET'], name="ETKDGctab23D")
    def ETKDGctab23D(ctab):
        """
Generate 3D coordinates from molfile using Experimental-Torsion "basic Knowledge" Distance Geometry.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ETKDGctab23D/$(cat no_coords.mol | base64 -w 0 | tr "+/" "-_")
        """

        data = base64.urlsafe_b64decode(ctab)
        return ETKDGctab23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/ETKDGctab23D', method=['OPTIONS', 'POST'], name="ETKDGctab23D")
    def ETKDGctab23D():
        """
Generate 3D coordinates from molfile using Experimental-Torsion "basic Knowledge" Distance Geometry.
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @no_coords.mol ${BEAKER_ROOT_URL}ETKDGctab23D
    curl -X POST -F "file=@no_coords.mol" ${BEAKER_ROOT_URL}ETKDGctab23D    
        """

        data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
        return ETKDGctab23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    def ETKDGsmiles23DView(data, params):
        kwargs = dict()
        kwargs['multi'] = int(params.get('multi', False))
        kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', False))
        kwargs['delimiter'] = params.get('delimiter', ' ')
        kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
        kwargs['nameColumn'] = int(params.get('nameColumn', 1))
        kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
        kwargs['mmff'] = True

        if params.get('titleLine') is None and not data.startswith('SMILES Name'):
            kwargs['titleLine'] = False
        else:
            kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

        return _smiles23D(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/ETKDGsmiles23D/<ctab>', method=['OPTIONS', 'GET'], name="ETKDGsmiles23D")
    def ETKDGsmiles23D(ctab):
        """
Generate 3D coordinates from SMILES using Experimental-Torsion "basic Knowledge" Distance Geometry.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}ETKDGsmiles23D/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}ETKDGsmiles23D/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")    
        """

        data = base64.urlsafe_b64decode(ctab)
        return ETKDGsmiles23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/ETKDGsmiles23D', method=['OPTIONS', 'POST'], name="ETKDGsmiles23D")
    def ETKDGsmiles23D():
        """
Generate 3D coordinates from SMILES using Experimental-Torsion "basic Knowledge" Distance Geometry.
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin_with_header.smi ${BEAKER_ROOT_URL}ETKDGsmiles23D
    curl -X POST -F "file=@aspirin_with_header.smi" ${BEAKER_ROOT_URL}ETKDGsmiles23D
    curl -X POST --data-binary @aspirin_no_header.smi ${BEAKER_ROOT_URL}ETKDGsmiles23D
    curl -X POST -F "file=@aspirin_no_header.smi" ${BEAKER_ROOT_URL}ETKDGsmiles23D
        """

        data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
        return ETKDGsmiles23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------
if hasattr(AllChem, 'KDG'):

    # TODO: this one is redundant (but you have to remember about conditional availability)
    def KDGctab23DView(data, params):
        kwargs = dict()
        kwargs['multi'] = int(params.get('multi', False))
        kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
        kwargs['removeHs'] = _parseFlag(params.get('removeHs', True))
        kwargs['strictParsing'] = _parseFlag(params.get('strictParsing', True))
        kwargs['mmff'] = True

        return _ctab23D(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/KDGctab23D/<ctab>', method=['OPTIONS', 'GET'], name="KDGctab23D")
    def KDGctab23D(ctab):
        """
Generate 3D coordinates from molfile using "basic Knowledge" Distance Geometry.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}KDGctab23D/$(cat no_coords.mol | base64 -w 0 | tr "+/" "-_")
        """

        data = base64.urlsafe_b64decode(ctab)
        return KDGctab23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/KDGctab23D', method=['OPTIONS', 'POST'], name="KDGctab23D")
    def KDGctab23D():
        """
Generate 3D coordinates from molfile using "basic Knowledge" Distance Geometry.
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @no_coords.mol ${BEAKER_ROOT_URL}KDGctab23D
    curl -X POST -F "file=@no_coords.mol" ${BEAKER_ROOT_URL}KDGctab23D    
        """

        data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
        return KDGctab23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    def KDGsmiles23DView(data, params):
        kwargs = dict()
        kwargs['multi'] = int(params.get('multi', False))
        kwargs['computeCoords'] = _parseFlag(params.get('computeCoords', False))
        kwargs['delimiter'] = params.get('delimiter', ' ')
        kwargs['smilesColumn'] = int(params.get('smilesColumn', 0))
        kwargs['nameColumn'] = int(params.get('nameColumn', 1))
        kwargs['sanitize'] = _parseFlag(params.get('sanitize', True))
        kwargs['mmff'] = True

        if params.get('titleLine') is None and not data.startswith('SMILES Name'):
            kwargs['titleLine'] = False
        else:
            kwargs['titleLine'] = _parseFlag(params.get('titleLine', True))

        return _smiles23D(data, **kwargs)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/KDGsmiles23D/<ctab>', method=['OPTIONS', 'GET'], name="KDGsmiles23D")
    def KDGsmiles23D(ctab):
        """
Generate 3D coordinates from SMILES using "basic Knowledge" Distance Geometry.
CTAB is urlsafe_base64 encoded string containing single molfile or concatenation of multiple molfiles.
cURL examples:

    curl -X GET ${BEAKER_ROOT_URL}KDGsmiles23D/$(cat aspirin_with_header.smi | base64 -w 0 | tr "+/" "-_")
    curl -X GET ${BEAKER_ROOT_URL}KDGsmiles23D/$(cat aspirin_no_header.smi | base64 -w 0 | tr "+/" "-_")    
        """

        data = base64.urlsafe_b64decode(ctab)
        return KDGsmiles23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------

    @app.route('/KDGsmiles23D', method=['OPTIONS', 'POST'], name="KDGsmiles23D")
    def KDGsmiles23D():
        """
Generate 3D coordinates from SMILES using "basic Knowledge" Distance Geometry.
CTAB is either single molfile or SDF file.
cURL examples:

    curl -X POST --data-binary @aspirin_with_header.smi ${BEAKER_ROOT_URL}KDGsmiles23D
    curl -X POST -F "file=@aspirin_with_header.smi" ${BEAKER_ROOT_URL}KDGsmiles23D
    curl -X POST --data-binary @aspirin_no_header.smi ${BEAKER_ROOT_URL}KDGsmiles23D
    curl -X POST -F "file=@aspirin_no_header.smi" ${BEAKER_ROOT_URL}KDGsmiles23D
        """

        data = request.files.values()[0].file.read() if len(request.files) else request.body.read()
        return KDGsmiles23DView(data, request.params)

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from rdkit import Chem
from rdkit.Chem.rdmolops import SanitizeFlags as sf
SANITIZE_ALL = sf.SANITIZE_ALL
from chembl_beaker.beaker.utils.functional import _apply, _call
from chembl_beaker.beaker.utils.io import _parseMolData, _getSDFString

#-----------------------------------------------------------------------------------------------------------------------

def _atomRings(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    return _call(_call(mols, 'GetRingInfo'), 'AtomRings')

#-----------------------------------------------------------------------------------------------------------------------

def _bondRings(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    return _call(_call(mols, 'GetRingInfo'), 'BondRings')

#-----------------------------------------------------------------------------------------------------------------------

def _isAtomInRing(data, index, size, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    return _call(_call(mols, 'GetRingInfo'), 'IsAtomInRingOfSize', index, size)

#-----------------------------------------------------------------------------------------------------------------------

def _isBondInRing(data, index, size, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    return _call(_call(mols, 'GetRingInfo'), 'IsBondInRingOfSize', index, size)

#-----------------------------------------------------------------------------------------------------------------------

def _numAtomRings(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    ring_infos = _call(mols, 'GetRingInfo')
    return [[ring_info.NumAtomRings(atom.GetIdx()) for atom in mol.GetAtoms()] for (mol, ring_info) in zip(mols, ring_infos)]

#-----------------------------------------------------------------------------------------------------------------------

def _numBondRings(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    ring_infos = _call(mols, 'GetRingInfo')
    return [[ring_info.NumBondRings(bond.GetIdx()) for bond in mol.GetBonds()] for (mol, ring_info) in zip(mols, ring_infos)]

#-----------------------------------------------------------------------------------------------------------------------

def _numRings(data, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    return _call(_call(mols, 'GetRingInfo'), 'NumRings')

#-----------------------------------------------------------------------------------------------------------------------



# $Id$
#
# Copyright (C) 2006-2011 Greg Landrum
#  All Rights Reserved
#
import os
from chembl_beaker.beaker.draw.molDrawing import MolDrawing, DrawingOptions


# ----------------------------------------------------------------------------------------------------------------------


def CoordsAreAllZero(m, confId=-1):
    conf = m.GetConformer(confId)
    for i in range(m.GetNumAtoms()):
        if list(conf.GetAtomPosition(i)) != [0.0, 0.0, 0.0]:
            return False
    return True


# ----------------------------------------------------------------------------------------------------------------------


def _getCanvas():
    useAGG = False
    useCairo = True
    from chembl_beaker.beaker.draw.cairoCanvas import Canvas
    return useAGG, useCairo, Canvas


# ----------------------------------------------------------------------------------------------------------------------


def _createCanvas(size):
    useAGG, useCairo, Canvas = _getCanvas()
    from PIL import Image
    img = Image.new("RGBA", size, (0, 0, 0, 0))
    canvas = Canvas(img)
    return img, canvas


# ----------------------------------------------------------------------------------------------------------------------


def MolToImage(mol, size=(300, 300), kekulize=True, wedgeBonds=True,
               fitImage=False, options=None, canvas=None, **kwargs):
    """ returns a PIL image containing a drawing of the molecule

    Keyword arguments:
    kekulize -- run kekulization routine on input `mol` (default True)
    size -- final image size, in pixel (default (300,300))
    wedgeBonds -- draw wedge (stereo) bonds (default True)
    highlightAtoms -- list of atoms to highlight (default [])
    highlightMap -- dictionary of (atom, color) pairs (default None)
    highlightBonds -- list of bonds to highlight (default [])
    """

    if not mol:
        raise ValueError('Null molecule provided')
    if canvas is None:
        img, canvas = _createCanvas(size)
    else:
        img = None

    fontSize = int(min(size) / 20)
    if min(size) < 200:
        fontSize = 1

    if options is None:
        options = DrawingOptions()
    if fitImage:
        options.dotsPerAngstrom = int(min(size) / 10)
    options.wedgeDashedBonds = wedgeBonds
    options.atomLabelFontSize = fontSize
    drawer = MolDrawing(canvas=canvas, drawingOptions=options)

    if kekulize:
        from rdkit import Chem
        Chem.Kekulize(mol)

    if not mol.GetNumConformers() or CoordsAreAllZero(mol):
        from rdkit.Chem import AllChem
        AllChem.Compute2DCoords(mol)

    if 'legend' in kwargs:
        legend = kwargs['legend']
        del kwargs['legend']
    else:
        legend = ''

    drawer.AddMol(mol, **kwargs)

    if legend:
        from chembl_beaker.beaker.draw.molDrawing import Font
        bbox = drawer.boundingBoxes[mol]
        pos = size[0] / 2, int(.94 * size[1]), 0  # the 0.94 is extremely empirical
        # canvas.addCanvasPolygon(((bbox[0],bbox[1]),(bbox[2],bbox[1]),(bbox[2],bbox[3]),(bbox[0],bbox[3])),
        #                         color=(1,0,0),fill=False,stroke=True)
        # canvas.addCanvasPolygon(((0,0),(0,size[1]),(size[0],size[1]),(size[0],0)   ),
        #                         color=(0,0,1),fill=False,stroke=True)
        font = Font(face='sans', size=12)
        canvas.addCanvasText(legend, pos, font)

    if kwargs.get('returnCanvas', False):
        return img, canvas, drawer
    else:
        canvas.flush()
        return img


# ----------------------------------------------------------------------------------------------------------------------


def MolToFile(mol, fileName, size=(300, 300), kekulize=True, wedgeBonds=True,
              imageType=None, fitImage=False, options=None, **kwargs):
    """ Generates a drawing of a molecule and writes it to a file
    """
    # original contribution from Uwe Hoffmann
    if not fileName:
        raise ValueError('no fileName provided')
    if not mol:
        raise ValueError('Null molecule provided')

    if imageType is None:
        imageType = os.path.splitext(fileName)[1][1:]

    if options is None:
        options = DrawingOptions()
    useAGG, useCairo, Canvas = _getCanvas()
    if fitImage:
        options.dotsPerAngstrom = int(min(size) / 10)
    options.wedgeDashedBonds = wedgeBonds
    if useCairo or useAGG:
        canvas = Canvas(size=size, imageType=imageType, fileName=fileName)
    else:
        options.radicalSymbol = '.'  # <- the sping canvas doesn't support unicode well
        canvas = Canvas(size=size, name=fileName, imageType=imageType)
    drawer = MolDrawing(canvas=canvas, drawingOptions=options)
    if kekulize:
        from rdkit import Chem
        mol = Chem.Mol(mol.ToBinary())
        Chem.Kekulize(mol)

    if not mol.GetNumConformers():
        from rdkit.Chem import AllChem
        AllChem.Compute2DCoords(mol)

    drawer.AddMol(mol, **kwargs)
    if useCairo or useAGG:
        canvas.flush()
    else:
        canvas.save()


# ----------------------------------------------------------------------------------------------------------------------


def MolToImageFile(mol, filename, size=(300, 300), kekulize=True, wedgeBonds=True, **kwargs):
    """  DEPRECATED:  please use MolToFile instead

     """
    img = MolToImage(mol, size=size, kekulize=kekulize, wedgeBonds=wedgeBonds, **kwargs)
    img.save(filename)


# ----------------------------------------------------------------------------------------------------------------------


tkRoot = None
tkLabel = None
tkPI = None


# ----------------------------------------------------------------------------------------------------------------------


def ShowMol(mol, size=(300, 300), kekulize=True, wedgeBonds=True, title='RDKit Molecule', **kwargs):
    """ Generates a picture of a molecule and displays it in a Tkinter window
    """
    global tkRoot, tkLabel, tkPI
    import Tkinter
    try:
        import ImageTk
    except ImportError:
        from PIL import ImageTk

    img = MolToImage(mol, size, kekulize, wedgeBonds, **kwargs)

    if not tkRoot:
        tkRoot = Tkinter.Tk()
        tkRoot.title(title)
        tkPI = ImageTk.PhotoImage(img)
        tkLabel = Tkinter.Label(tkRoot, image=tkPI)
        tkLabel.place(x=0, y=0, width=img.size[0], height=img.size[1])
    else:
        tkPI.paste(img)
    tkRoot.geometry('%dx%d' % img.size)


# ----------------------------------------------------------------------------------------------------------------------


def calcAtomGaussians(mol, a=0.03, step=0.02, weights=None):
    import numpy
    from matplotlib import mlab
    x = numpy.arange(0, 1, step)
    y = numpy.arange(0, 1, step)
    X, Y = numpy.meshgrid(x, y)
    if weights is None:
        weights = [1.] * mol.GetNumAtoms()
    Z = mlab.bivariate_normal(X, Y, a, a, mol._atomPs[0][0], mol._atomPs[0][1]) * weights[0]
    for i in range(1, mol.GetNumAtoms()):
        Zp = mlab.bivariate_normal(X, Y, a, a, mol._atomPs[i][0], mol._atomPs[i][1])
        Z += Zp * weights[i]
    return X, Y, Z


# ----------------------------------------------------------------------------------------------------------------------


def MolsToImage(mols, subImgSize=(200, 200), legends=None, **kwargs):
    """ 
    """
    try:
        import Image
    except ImportError:
        from PIL import Image
    if legends is None:
        legends = [None] * len(mols)
    res = Image.new("RGBA", (subImgSize[0] * len(mols), subImgSize[1]))
    for i, mol in enumerate(mols):
        res.paste(MolToImage(mol, subImgSize, legend=legends[i], **kwargs), (i * subImgSize[0], 0))
    return res


# ----------------------------------------------------------------------------------------------------------------------


def MolsToGridImage(mols, molsPerRow=3, subImgSize=(200, 200), legends=None, highlightAtomLists=None, **kwargs):
    """ 
    """
    try:
        import Image
    except ImportError:
        from PIL import Image
    if legends is None:
        legends = [None] * len(mols)

    nRows = len(mols) // molsPerRow
    if len(mols) % molsPerRow:
        nRows += 1

    res = Image.new("RGBA", (molsPerRow * subImgSize[0], nRows * subImgSize[1]), (255, 255, 255, 0))
    for i, mol in enumerate(mols):
        row = i // molsPerRow
        col = i % molsPerRow
        highlights = None
        if highlightAtomLists and highlightAtomLists[i]:
            highlights = highlightAtomLists[i]
        res.paste(MolToImage(mol, subImgSize, legend=legends[i], highlightAtoms=highlights, fitImage=True, **kwargs),
                  (col * subImgSize[0], row * subImgSize[1]))
    return res


# ----------------------------------------------------------------------------------------------------------------------


def ReactionToImage(rxn, subImgSize=(200, 200), **kwargs):
    """ 
    """
    try:
        import Image
    except ImportError:
        from PIL import Image

    mols = []
    for i in range(rxn.GetNumReactantTemplates()):
        tmpl = rxn.GetReactantTemplate(i)
        tmpl.UpdatePropertyCache(False)
        mols.append(tmpl)
    mols.append(None)
    for i in range(rxn.GetNumProductTemplates()):
        tmpl = rxn.GetProductTemplate(i)
        tmpl.UpdatePropertyCache(False)
        mols.append(tmpl)

    res = Image.new("RGBA", (subImgSize[0] * len(mols), subImgSize[1]), (255, 255, 255, 0))
    for i, mol in enumerate(mols):
        if mol is not None:
            nimg = MolToImage(mol, subImgSize, kekulize=True, **kwargs)
        else:
            nimg, canvas = _createCanvas(subImgSize)
            p0 = (10, subImgSize[1] // 2)
            p1 = (subImgSize[0] - 10, subImgSize[1] // 2)
            p3 = (subImgSize[0] - 20, subImgSize[1] // 2 - 10)
            p4 = (subImgSize[0] - 20, subImgSize[1] // 2 + 10)
            canvas.addCanvasLine(p0, p1, lineWidth=2, color=(0, 0, 0))
            canvas.addCanvasLine(p3, p1, lineWidth=2, color=(0, 0, 0))
            canvas.addCanvasLine(p4, p1, lineWidth=2, color=(0, 0, 0))
            if hasattr(canvas, 'flush'):
                canvas.flush()
            else:
                canvas.save()
        res.paste(nimg, (i * subImgSize[0], 0))
    return res

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

from rdkit import Chem
from rdkit.Chem import AllChem
from chembl_beaker.beaker.utils.functional import _apply
from chembl_beaker.beaker.utils.io import _parseMolData, _getSDFString, _parseSMILESData

# ----------------------------------------------------------------------------------------------------------------------


def _2D23D(mol, multi, mmff=False, embed=None):
    molH = Chem.AddHs(mol)
    if embed:
        if embed == 'ETKDG':
            AllChem.EmbedMolecule(molH, AllChem.ETKDG())
        elif embed == 'KDG':
            AllChem.EmbedMolecule(molH, AllChem.KDG())
        return molH
    if mmff:
        optiFunc = AllChem.MMFFOptimizeMolecule
    else:
        optiFunc = AllChem.UFFOptimizeMolecule
    if multi:
        confIds=AllChem.EmbedMultipleConfs(molH, multi)
        for confId in confIds:
          optiFunc(molH, confId=confId)
    else:
        AllChem.EmbedMolecule(molH)
        optiFunc(molH)
    return molH

# ----------------------------------------------------------------------------------------------------------------------


def _ctab23D(data, multi, mmff, sanitize=True, removeHs=True, strictParsing=True):
    mols = _parseMolData(data, sanitize=sanitize, removeHs=removeHs, strictParsing=strictParsing)
    optimisedMols = _apply(mols, _2D23D, multi, mmff)
    return _getSDFString(optimisedMols)

# ----------------------------------------------------------------------------------------------------------------------


def _smiles23D(data, multi, mmff, computeCoords=False, delimiter=' ', smilesColumn=0, nameColumn=1,
               titleLine=True, sanitize=True):
    mols = _parseSMILESData(data, computeCoords=computeCoords, delimiter=delimiter, smilesColumn=smilesColumn,
        nameColumn=nameColumn, titleLine=titleLine, sanitize=sanitize)
    optimisedMols = _apply(mols, _2D23D, multi, mmff)
    return _getSDFString(optimisedMols)

# ----------------------------------------------------------------------------------------------------------------------
